package com.autonavi.alinkclient;

import android.content.Context;

import com.autonavi.alinkclient.listener.EventListener;
import com.autonavi.alinkclient.listener.RtpListener;
import com.autonavi.alinkclient.model.TouchEvent;

/**
 * AlinkClientSDK的入口类
 *
 * @author alink
 */
public class AlinkClient {

    /**
     * AlinkClient的内部封装类
     */
    private AlinkClientWrapper mAlinkClientWrapper = new AlinkClientWrapper();
    /**
     * android进程的运行环境
     */
    private Context mContext;

    /**
     * 初始化过程,设置sdk运行需要用到的参数
     *
     * @param context android进程的运行环境
     * @param config  Alink运行的音视频参数.
     *                其中视频参数包括:编码宽度,高度,密度,帧率;
     *                音频参数包括:播放navigation的streamType,播放music的streamType,播放speech的streamType
     */
    public void onCreate(Context context, AlinkConfig config) {
        if (context == null || config == null) {
            return;
        }
        mContext = context;
        mAlinkClientWrapper.onCreate(context);
        mAlinkClientWrapper.config(config);
    }

    /**
     * 注册状态监听类
     *
     * @param listener 状态监听类
     */
    public void registerStatusListener(OnConnectListener listener) {
        mAlinkClientWrapper.registerConnectListener(listener);
    }

    /**
     * 注册事件监听类
     *
     * @param listener 　事件监听类
     */
    public void registerEventListener(EventListener listener) {
        mAlinkClientWrapper.registerEventListener(listener);
    }

    /**
     * 注册音视频类
     *
     * @param type     　RTP流的类型
     * @param listener 　事件监听类
     */
    public void registerAVListener(int type, RtpListener listener) {
        mAlinkClientWrapper.registerAVListener(type, listener);
    }

    /**
     * 连接AlinkServer
     *
     * @param type 指定的连接方式: usb or wifi
     */
    public void connect(int type) {
        if (ConnectType.CONNECT_TYPE_USB == type) {
            mAlinkClientWrapper.startConnectByUsb();
        } else if (ConnectType.CONNECT_TYPE_WIFI == type) {
            mAlinkClientWrapper.startConnectByWifi();
        } else {
            // empty by design
        }
    }

    /**
     * 发送指令
     *
     * @param command 以json格式封装的指令信息, 目前支持以下2个json串内容
     *                {"command":"start"} 开始音视频流传输
     *                {"command":"stop"}　停止音视频流传输
     */
    public void postCommand(String command) {
        mAlinkClientWrapper.postCommand(command);
    }

    /**
     * 发送key事件
     */
    public void postKeyEvent(int keyCode, int keyAction) {
        mAlinkClientWrapper.postKeyEvent(keyCode, keyAction);
    }

    /**
     * 发送Touch事件
     */
    public void postTouchEvent(TouchEvent touchEvent) {
        mAlinkClientWrapper.postTouchEvent(touchEvent);
    }

    /**
     * 传送asr流数据
     */
    public void postASRData(byte[] data, int size) {
        mAlinkClientWrapper.sendASR(data, size);
    }

    /**
     * 断开AlinkServer
     */
    public void disconnect() {
        mAlinkClientWrapper.disconnect();
    }

    /**
     * 反注册状态监听
     */
    public void unregisterStatusListener() {
        mAlinkClientWrapper.registerConnectListener(null);
    }

    /**
     * 取消事件的监听
     */
    public void unregisterEventListener() {
        mAlinkClientWrapper.registerEventListener(null);
    }

    /**
     * 销毁AlinkClientsdk
     */
    public void onDestroy() {
        mAlinkClientWrapper.onDestroy(mContext);
    }

    /**
     * 状态监听接口
     */
    public interface OnConnectListener {
        /**
         * 回调通知连接的状态
         *
         * @param status 状态类型: usb连接成功, usb连接断开, wifi连接成功, wifi连接断开
         */
        void onConnectStatus(int status);

        public final class Status {
            /**
             * 未连接
             */
            public static final int STATUS_ALINK_NONE_CONNECTED = 0;
            /**
             * usb连接成功
             */
            public static final int STATUS_ALINK_USB_CONNECTED = 100;
            /**
             * usb连接断开
             */
            public static final int STATUS_ALINK_USB_DISCONNECTED = 200;
            /**
             * wifi连接成功
             */
            public static final int STATUS_ALINK_WIFI_CONNECTED = 300;
            /**
             * wifi连接断开
             */
            public static final int STATUS_ALINK_WIFI_DISCONNECTED = 400;
        }
    }

    /**
     * 由第三方应用传入的证书内容. 此处，传入的是证书内容，不是证书路径．
     */
    public static class CertConfig {
        /**
         * 证书校验信息
         */
        public byte[] verifyInfo;

        /**
         * 证书内容
         */
        public byte[] certificate;

        /**
         * 私钥
         */
        public byte[] privateKey;
    }

    /**
     * Alink的参数信息
     */
    public static class AlinkConfig {
        /**
         * true: 当前仅支持使用sdk中的播放控件，第三方应用不需要再注册音视频流的监听类，当clientsdk接收到音视频流时会直接展示
         */
        public boolean usePlayerInSDK;

        /**
         * 视频编码宽度,800是缺省值，第三方应用中可以传入实际需要的值，该值会传到server端
         */
        public int codecWidth = 800;

        /**
         * 视频编码高度,480是缺省值，第三方应用中可以传入实际需要的值，该值会传到server端
         */
        public int codecHeight = 480;

        /**
         * 视频编码密度,160是缺省值，第三方应用中可以传入实际需要的值，该值会传到server端
         */
        public int codecDpi = 160;

        /**
         * 视频编码帧率,30是缺省值，第三方应用中可以传入实际需要的值，该值会传到server端
         */
        public int codecFrameRate = 30;

        /**
         * 导航播放类型, 取值STREAM_RING
         */
        public int audioNavigationStreamType;

        /**
         * 音乐播放类型, 取值STREAM_MUSIC
         */
        public int audioMusicStreamType;

        /**
         * 语音播放类型, 取值STREAM_RING
         */
        public int audioSpeechStreamType;

        /**
         * 证书信息
         */
        public CertConfig certConfig;
    }

    public static class KeyCode {
        /**
         * asr停止键值
         */
        public static final int KEYCODE_ASR_STOP = 0x00000056;

        /**
         * asr开始键值
         */
        public static final int KEYCODE_ASR_START = 0x00000115;
    }

    public static class ButtonState {
        /**
         * 按钮按下
         */
        public static final int BUTTONSTATE_PRESSED = 0x0000000B;

        /**
         * 按钮抬起
         */
        public static final int BUTTONSTATE_RELEASED = 0x0000000C;
    }

    public static class ConnectType {
        /**
         * 连接类型为usb
         */
        public static final int CONNECT_TYPE_USB = 100;

        /**
         * 连接类型为wifi
         */
        public static final int CONNECT_TYPE_WIFI = 200;
    }

    public static class RtpType {
        /**
         * RTP流的类型为video
         */
        public static final int RTP_VIDEO = 1;

        /**
         * RTP流的类型为music
         */
        public static final int RTP_MUSIC = 2;

        /**
         * RTP流的类型为navigation
         */
        public static final int RTP_NAVIGATION = 3;

        /**
         * RTP流的类型为speech
         */
        public static final int RTP_SPEECH = 4;
    }
}
